/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 Fossil E-Commerce, http://www.fossilec.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: ServiceLifeCycle.java 292 2006-05-02 07:30:43Z ddesjardins $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals;


/**
 * This interface must be implemented by internal platform services to provide
 * initialization, start, stop and shutdown life cycle processing. These methods
 * comprise the life cycle contract between platform services and the management
 * functionalities.
 * 
 * @version $Rev: 292 $ $Date: 2006-05-02 07:30:43Z $
 * @since Petals 1.0
 * @author <a href="mailto:rmarins@fossilec.com">Rafael Marins</a>
 * TODO check if it is necessary to keep this class
 */
public interface ServiceLifeCycle {

	/**
	 * Value returned by {@link ServiceLifeCycle#getCurrentState()} for a
	 * shutdown system service.
	 */
	public static final java.lang.String SHUTDOWN = "Shutdown";

	/**
	 * Value returned by {@link ServiceLifeCycle#getCurrentState()} for a
	 * started system service.
	 */
	public static final java.lang.String STARTED = "Started";

	/**
	 * Value returned by {@link ServiceLifeCycle#getCurrentState()} for a
	 * stopped platform service.
	 */
	public static final java.lang.String STOPPED = "Stopped";

	/**
	 * Value returned by {@link ServiceLifeCycle#getCurrentState()} for a
	 * platform service in an unknown state.
	 */
	public static final java.lang.String UNKNOWN = "Unknown";

	/**
	 * Get the current state of this managed platform service.
	 * 
	 * @return the current state of this managed service (must be one of
	 *         the string constants defined by this interface)
	 */
	public java.lang.String getCurrentState();

	/**
	 * Initialize the platform service. This performs initialization required by
	 * the service but does not make it ready to get working. This method is
	 * called once for each life cycle of the platform service.
	 * <p>
	 * The current service state becomes {@link ServiceLifeCycle#STOPPED}
	 * just after this method invocation.
	 * 
	 * @param context
	 *            the system service's context, providing access to service's
	 *            data provided by Petals environemnt.
	 * @throws PetalsException
	 */
	public void init(ServiceContext context) throws PetalsException;

	/**
	 * Shut down the platform service. This performs clean-up, releasing all
	 * run-time resources used by the service. Once this method has been
	 * called, {@link ServiceLifeCycle#init(ServiceContext)} must be called
	 * before the service can be started again with a call to
	 * {@link ServiceLifeCycle#start()}.
	 * 
	 * @throws PetalsException
	 *             if the system service fails to shut down
	 */
	public void shutDown() throws PetalsException;

	/**
	 * Start the platform service. This makes the service ready to use by other
	 * services. This method is called after
	 * {@link ServiceLifeCycle#init(ServiceContext)}, both when the service is
	 * being started for the first time and when the service is being restarted
	 * after a previous call to {@link ServiceLifeCycle#shutDown()}. If
	 * {@link ServiceLifeCycle#stop()} was called previously but
	 * {@link ServiceLifeCycle#shutDown()} was not,
	 * {@link ServiceLifeCycle#start()} can be called again without another call
	 * to {@link ServiceLifeCycle#init(ServiceContext)}.
	 * 
	 * @throws PetalsException
	 *             if the system service fails to start
	 */
	public void start() throws PetalsException;

	/**
	 * Stop the platform service. This makes the service stop accepting calls
	 * for processing. After a call to this method,
	 * {@link ServiceLifeCycle#start()} may be called again without first
	 * calling {@link ServiceLifeCycle#init(ServiceContext)}.
	 * 
	 * @throws PetalsException
	 *             if the system services fails to stop
	 */
	public void stop() throws PetalsException;

}
