/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 Fossil E-Commerce, http://www.fossilec.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: ClassPath.java 98 2006-02-24 16:18:48Z gblondelle $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.classloader;

import java.io.File;
import java.io.FileFilter;
import java.io.FilenameFilter;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;


/**
 * @author Rafael Marins
 * @version $Rev: 98 $ $Date: 2006-02-24 16:18:48Z $
 */
public class ClassPath {

	private static class DirectoryFilter implements FileFilter {
	
		/**
		 * @see java.io.FileFilter#accept(java.io.File)
		 */
		public boolean accept(File pathname) {
			return pathname.isDirectory();
		}
		
	}

	private static class ExtensionFileFilter implements FilenameFilter {
	
		private String[] extensions;
	
		public ExtensionFileFilter(String[] exts) {
			super();
			this.extensions = exts;
		}
	
		/**
		 * @see java.io.FilenameFilter#accept(java.io.File, java.lang.String)
		 */
		public boolean accept(File dir, String name) {
	
			boolean accept = false;
	
			String lcName = name.toLowerCase();
	
			for (int i=0; i<extensions.length; i++) {
				String x = extensions[i];
				if (lcName.endsWith(x)) {
					accept = true;
					break;
				}
			}
	
			return accept;
		}
		
	}

	private List<File> pathComponents = new ArrayList<File>();

	private boolean recursiveScanner = false;

	/**
	 * 
	 */
	public ClassPath() {
		super();
	}

	public ClassPath(boolean recursive) {
		super();
		this.recursiveScanner = recursive;
	}

	public void setClasspath(String classpath) {
		pathComponents.clear();

		String[] elements = classpath.split(File.pathSeparator);
		setClasspath(elements);
	}

	public void setClasspath(String[] classpath) {
		for (int i=0; i<classpath.length; i++) {
			addPathElement(classpath[i]);
		}
	}

	public void addPathElement(String path) {
		addPathElement(path, recursiveScanner, null);
	}

	public void addPathElement(String path, boolean recursive,
			String[] extensions) {

		File pathComponent = new File(path);
		addPathFile(pathComponent, recursive, extensions);
	}

	public void addPathFile(File component, boolean recursive) {
		addPathFile(component, recursive, null);
	}

	public void addPathFile(File pathElement, boolean recursive,
			String[] extensions) {

		if (pathElement == null) {
			throw new IllegalArgumentException("Classpath component must not " +
					"be null");
		}

		// Logic for adding a directory into classpath
		if (pathElement.isDirectory()) {
			Collection<File> components;

			if (recursive) {
				components = scanComponentsRecursively(pathElement,
						extensions);
			} else {
				components = scanComponents(pathElement, extensions);
			}

			for (File file : components) {
				pathComponents.add(file);
			}
		// Logic for adding a single file into classpath
		} else {
			pathComponents.add(pathElement);
		}
	}

	private Collection<File> scanComponentsRecursively(File element,
			String[] extensions) {

		Collection<File> componentsFound = new ArrayList<File>();

		componentsFound.addAll(scanComponents(element, extensions));

		File[] directories = element.listFiles(new DirectoryFilter());

		for (int i=0; i<directories.length; i++) {
			File d = directories[i];
			componentsFound.addAll(scanComponentsRecursively(d, extensions));
		}

		return componentsFound;
	}

	private Collection<File> scanComponents(File element,
			String[] extensions) {

		Collection<File> componentsFound = new ArrayList<File>();

		if (extensions == null) {
			componentsFound.add(element);

		} else {
			File[] elements = element.listFiles(
					new ExtensionFileFilter(extensions));

			componentsFound.addAll(Arrays.asList(elements));
		}

		return componentsFound;
	}

	/**
	 * @see java.lang.Iterable#iterator()
	 */
	public Iterator iterator() {
		return pathComponents.iterator();
	}

	/**
	 * @see java.util.Vector#size()
	 */
	public int size() {
		return pathComponents.size();
	}

	/**
	 * @see java.util.Vector#elementAt(int)
	 */
	public File elementAt(int index) {
		return (File) pathComponents.get(index);
	}

}
