/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: ComponentClassLoaderTest.java 154 2006-03-27 15:30:10Z ofabre $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.classloader;

import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import java.util.List;

/**
 * @author ofabre - eBMWebsourcing
 * @author ddesjardins - eBMWebsourcing
 */
public class ComponentClassLoader extends PetalsClassLoader {

	/**
	 * List of the sharedlibraries that must be included in the classloading
	 * strategy.
	 */
	private List<String> sharedLibrariesNameList;

	private SharedLibrariesClassLoader sharedLibrariesClassLoader;

	public ComponentClassLoader(URL[] baseUrls, List<String> classpathLocations, SharedLibrariesClassLoader parent, List<String> sharedLibrariesNameList) throws IOException {
		super(baseUrls, classpathLocations, parent);

		this.sharedLibrariesNameList = sharedLibrariesNameList;

		this.sharedLibrariesClassLoader = parent;
	}

	/**
	 * If class is not found, search in the sharedLibrariesClassLoader, by
	 * specifying the sharedLibraries to use.
	 * 
	 * @see java.lang.ClassLoader#loadClass(java.lang.String, boolean)
	 */
	@Override
	protected synchronized Class<?> loadClass(String className, boolean resolve) throws ClassNotFoundException {
		Class<?> clazz = null;
		clazz = findLoadedClass(className);
		if (clazz == null) {
			// parent first
			if (useParentFirst(className, true)) {
				try {
					clazz = sharedLibrariesClassLoader.loadClass(className, resolve, sharedLibrariesNameList);
				}
				catch (ClassNotFoundException cnfe) {
					clazz = findClass(className);
				}
			}
			// self first
			else {
				try {
					clazz = findClass(className);
				}
				catch (ClassNotFoundException cnfe) {
					if (isIsolated()) {
						throw cnfe;
					}
					else {
						clazz = sharedLibrariesClassLoader.loadClass(className, resolve, sharedLibrariesNameList);
					}
				}
			}
		}
		if (resolve) {
			resolveClass(clazz);
		}
		return clazz;
	}

	/**
	 * If resource is not found, search in the sharedLibrariesClassLoader, by
	 * specifying the sharedLibraries to use.
	 * 
	 * @see java.lang.ClassLoader#getResource(java.lang.String)
	 */
	@Override
	public synchronized URL getResource(String resourceName) {
		URL url = null;
		// parent first
		if (isParentFirst()) {
			url = sharedLibrariesClassLoader.getResource(resourceName, sharedLibrariesNameList);
			if (url == null){
				// Try to search in the parent classpath
				url = super.getResource(resourceName);
			}
		}
		// self first
		else {
			url = super.getResource(resourceName);
			if (url == null){
				// Try to search in its own classpath
				url = sharedLibrariesClassLoader.getResource(resourceName, sharedLibrariesNameList);
			}
		}
		return url;
	}
	
	/**
	 * If resource is not found, search in the sharedLibrariesClassLoader, by
	 * specifying the sharedLibraries to use.
	 * 
	 * @see java.lang.ClassLoader#getResourceAsStream(java.lang.String)
	 */
	@Override
	public synchronized InputStream getResourceAsStream(String resourceName) {
		InputStream inputStream = null;
		// parent first
		if (isParentFirst()) {
			inputStream = sharedLibrariesClassLoader.getResourceAsStream(resourceName, sharedLibrariesNameList);
			if (inputStream == null){
				// Try to search in the parent classpath
				inputStream = super.getResourceAsStream(resourceName);
			}
		}
		// self first
		else {
			inputStream = super.getResourceAsStream(resourceName);
			if (inputStream == null){
				// Try to search in its own classpath
				inputStream = sharedLibrariesClassLoader.getResourceAsStream(resourceName, sharedLibrariesNameList);
			}
		}
		return inputStream;
	}


}
