/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 Fossil E-Commerce, http://www.fossilec.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: LoaderManagerImpl.java 292 2006-05-02 07:30:43Z ddesjardins $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.classloader;

import java.io.IOException;
import java.net.URL;
import java.net.URLClassLoader;
import java.util.List;

import org.objectweb.petals.PetalsException;

/**
 * @version $Rev: 292 $ $Date: 2006-05-02 07:30:43Z $
 * @since Petals 1.0
 * @author <a href="mailto:rmarins@fossilec.com">Rafael Marins</a>, Adrien
 *         LOUIS
 */
public class LoaderManagerImpl implements LoaderManager {

	/**
     * Default constructor.
     */
	public LoaderManagerImpl() {
		super();
	}

	// ----------------------------------------------------------------------
	// ClassLoadingService interface implementation
	// ----------------------------------------------------------------------

	/**
     * Create a classLoader for a JBI Component. The parent classLoader will be
     * the SharedLibrariesClassLoader.
     * 
     * @see org.objectweb.petals.classloader.LoaderManager#createClassLoader(java.net.URL[],
     *      boolean)
     */
	public URLClassLoader createClassLoader(URL[] baseUrls, List<String> classpathLocations, boolean useParentFirst) throws PetalsException {

		// create the class loader
		PetalsClassLoader cl = null;

		Loaders loaders = Loaders.getInstance();

		try {
			cl = new PetalsClassLoader(baseUrls, classpathLocations, loaders.getSharedLibrariesClassLoader());

			cl.setParentFirst(useParentFirst);
		}
		catch (IOException e) {
			throw new PetalsException("Error while creating a class loader", e);
		}
		return cl;
	}

	/**
     * Remove classLoader from the list. If the classLoader is a ClassLoader for
     * sharedLibraries, remove it from the SharedLibrariesClassLoader
     * 
     * @see org.objectweb.petals.classloader.LoaderManager#deleteClassLoader(java.lang.String)
     */
	public void deleteClassLoader(String classLoaderId) {

		Loaders loaders = Loaders.getInstance();

		if (!containsClassLoader(classLoaderId)) {
			return;
		}

		// delete class loader for further retrieve
		loaders.remove(classLoaderId);
		loaders.getSharedLibrariesClassLoader().removeSharedLibrariesClassLoader(classLoaderId);
	}

	/**
     * @see org.objectweb.petals.classloader.LoaderManager#containsClassLoader(java.lang.String)
     */
	public boolean containsClassLoader(String classLoaderId) {

		Loaders loaders = Loaders.getInstance();

		return loaders.contains(classLoaderId);
	}

	/**
     * @see org.objectweb.petals.classloader.LoaderManager#createClassLoader(java.lang.String,
     *      java.net.URL[], boolean)
     */
	public URLClassLoader createComponentClassLoader(String classLoaderId, URL[] baseUrls, List<String> classpathUrls, boolean useParentFirst, List<String> sharedLibrariesNameList)
			throws PetalsException {

		// create the class loader
		PetalsClassLoader cl = null;

		Loaders loaders = Loaders.getInstance();

		try {
			cl = new ComponentClassLoader(baseUrls, classpathUrls, loaders.getSharedLibrariesClassLoader(), sharedLibrariesNameList);

			cl.setParentFirst(useParentFirst);
		}
		catch (IOException e) {
			throw new PetalsException("Error while creating a class loader", e);
		}

		// save for further retrieve (overwrite existing class loader)
		loaders.put(classLoaderId, cl);

		return cl;
	}

	/**
     * Create a classloader for a sharedLibrary. This classLoader will be added
     * to the SharedLibrariesClassLoader.
     * 
     * @see org.objectweb.petals.classloader.LoaderManager#createClassLoader(java.lang.String,
     *      java.net.URL[], boolean)
     */
	public URLClassLoader createSharedLibrariesClassLoader(String classLoaderId, URL[] baseUrls, List<String> classpathUrls, boolean useParentFirst) throws PetalsException {

		// create the class loader
		PetalsClassLoader cl = null;

		try {
			cl = new PetalsClassLoader(baseUrls, classpathUrls);
			cl.setParentFirst(useParentFirst);
		}
		catch (IOException e) {
			throw new PetalsException("Error while creating a class loader", e);
		}

		// save for further retrieve (overwrite existing class loader)
		Loaders loaders = Loaders.getInstance();

		loaders.put(classLoaderId, cl);
		loaders.getSharedLibrariesClassLoader().addSharedLibrariesClassLoader(classLoaderId, cl);

		return cl;
	}

}
