/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: PetalsClassLoader.java 296 2006-05-02 12:58:27Z ddesjardins $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.classloader;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import java.net.URLClassLoader;
import java.net.URLStreamHandlerFactory;
import java.util.Enumeration;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.jar.JarFile;

import org.objectweb.petals.classloader.factory.URLFactory;
import org.objectweb.petals.classloader.locator.Locator;

/**
 * Petals class loader Do not implement loadClass(String) it causes a conflict
 * with the SharedLibrariesClassLoader
 * 
 * @version $Rev: 296 $ $Date: 2006-05-02 12:58:27Z $
 * @since Petals 1.0
 * @author Rafael Marins, Olivier Fabre
 * @author ddesjardins - eBMWebsourcing
 */
public class PetalsClassLoader extends URLClassLoader {

	/**
     * Standard Java package binary name.
     */
	private static final String JAVA_PACKAGE = "java.";

	/**
     * Standard Java extensions package binary name.
     */
	private static final String JAVAX_PACKAGE = "javax.";

	/**
     * To use or not the parent first class loading delegation model.
     */
	private boolean parentFirst;

	/**
     * Isolate Class Loading from parent.
     */
	private boolean ignoreParent;

	/**
     * To use or not the package intelligence self-first mechanism.
     */
	private boolean usePackageIntelligence;

	/**
     * List of known packages loaded from this class loader.
     */
	private Set<String> localKnownPackages;

	/**
     * Parent Class Loader.
     */
	protected ClassLoader parent;

	/**
     * Factories used to create URLs for URLClassLoader
     */
	private URLFactory[] factories;

	/**
     * Locators used to search jar/dir for files, directories, ...
     */
	private Locator[] locators;

	/**
     * Bases URLs
     */
	private URL[] bases;

	/**
     * Create a class loader for petals
     * 
     * @param baseUrls installation root path in petals
     * @param classpathLocations jar defined in the classpath part of the jbi
     *        descriptor
     * @param parent
     * @throws IOException
     */
	public PetalsClassLoader(URL[] baseUrls, List<String> classpathLocations, ClassLoader parent) throws IOException {
		super(baseUrls, parent);
		this.parent = parent;
		this.bases = baseUrls;
		initialize(classpathLocations);
	}

	/**
     * Create a class loader for petals
     * 
     * @param baseUrls installation root path in petals
     * @param classpathLocations jar defined in the classpath part of the jbi
     *        descriptor
     * @throws IOException
     */
	public PetalsClassLoader(URL[] baseUrls, List<String> classpathLocations) throws IOException {
		super(baseUrls);
		this.bases = baseUrls;
		initialize(classpathLocations);
	}

	/**
     * Create a class loader for petals
     * 
     * @param baseUrls installation root path in petals
     * @param classpathLocations jar defined in the classpath part of the jbi
     *        descriptor
     * @param parent parent class loader
     * @param factory url factory
     * @throws IOException
     */
	public PetalsClassLoader(URL[] baseUrls, List<String> classpathLocations, ClassLoader parent, URLStreamHandlerFactory factory) throws IOException {
		super(baseUrls, parent, factory);
		this.parent = parent;
		this.bases = baseUrls;
		initialize(classpathLocations);
	}

	// ----------------------------------------------------------------------
	// Getters and Setters
	// ----------------------------------------------------------------------

	/**
     * @return Returns the useParentFirst.
     */
	public boolean isParentFirst() {
		return parentFirst;
	}

	/**
     * @param use The useParentFirst to set.
     */
	public void setParentFirst(boolean use) {
		this.parentFirst = use;
	}

	/**
     * @return Returns the isolated.
     */
	public boolean isIsolated() {
		return ignoreParent;
	}

	/**
     * @param isolated The ignoreParent to set.
     */
	public void setIsolated(boolean isolated) {
		this.ignoreParent = isolated;
	}

	/**
     * @return Returns the usePackageIntelligence.
     */
	public boolean isUsingPackageIntelligence() {
		localKnownPackages = null;
		return usePackageIntelligence;
	}

	/**
     * @param use The usePackageIntelligence to set.
     */
	public void setUsingPackageIntelligence(boolean use) {
		localKnownPackages = new HashSet<String>();
		this.usePackageIntelligence = use;
	}

	/**
     * @return Returns the bases.
     */
	public URL[] getBases() {
		return bases;
	}

	/**
     * @return Returns a String representation of the classpath used by this
     *         classloader
     */
	public String getClasspath() {
		URL[] urls = getURLs();
		StringBuffer cp = new StringBuffer();
		for (int i = 0; i < urls.length; i++) {
			String url = urls[i].getFile();
			// do not add URL with !/ inside
			if (url.indexOf("!/") == -1) {
				cp.append(File.pathSeparator + url);
			}
		}
		return cp.toString();
	}

	// ----------------------------------------------------------------------
	// overriding methods
	// ----------------------------------------------------------------------

	/**
     * Load a class Use the parent classloader if parentFirst is set to true
     * else use its own url classloader to load the class
     * 
     * @param className name of the class to load
     * @param resolve boolean to resolve or not the class
     */
	@Override
	protected synchronized Class<?> loadClass(String className, boolean resolve) throws ClassNotFoundException {
		Class<?> clazz = null;
		clazz = findLoadedClass(className);
		if (clazz == null) {
			// parent first
			if (useParentFirst(className, true)) {
				try {
					clazz = parent.loadClass(className);
				}
				catch (ClassNotFoundException cnfe) {
					clazz = findClass(className);
				}
			}
			// self first
			else {
				try {
					clazz = findClass(className);
				}
				catch (Exception cnfe) {
					try {
						clazz = parent.loadClass(className);
					}
					catch (Exception e) {
						throw new ClassNotFoundException(className);
					}
					if (clazz == null) {
						throw new ClassNotFoundException(className);
					}
				}
			}
		}
		if (resolve) {
			resolveClass(clazz);
		}
		return clazz;
	}

	/**
     * Find a class in the class path of the petals class loader
     * 
     * @param className name of the class to find
     * @throws ClassNotFoundException when fails (when the class is not found or
     *         when the class is a java.lang.object)
     */
	@Override
	protected Class<?> findClass(String className) throws ClassNotFoundException {
		return super.findClass(className);
	}

	/**
     * Return a URL of the resource
     * 
     * @param resourceName
     * @return url of the resource (null if the resource is not found)
     */
	@Override
	public URL getResource(String resourceName) {
		URL resource = null;
		// parent first
		if (useParentFirst(resourceName, false)) {
			resource = parent.getResource(resourceName);
			if (resource == null) {
				resource = findResource(resourceName);
			}
		}
		// self first
		else {
			resource = findResource(resourceName);
			if (resource == null && !isIsolated()) {
				resource = parent.getResource(resourceName);
			}
		}
		return resource;
	}

	/**
     * Get a stream from a resource in the classpath
     * 
     * @param resourceName name of the resource
     * @return stream of the resource (null if the resource is not found)
     */
	@Override
	public InputStream getResourceAsStream(String resourceName) {
		InputStream inputStream = null;
		// parent first
		if (useParentFirst(resourceName, false)) {
			inputStream = parent.getResourceAsStream(resourceName);
			if (inputStream == null) {
				URL url = findResource(resourceName);
				if (url != null) {
					try {
						if (url.getProtocol().equals("file")) {
							// The resource is a file
							inputStream = new FileInputStream(url.getPath());
						}
						else if (url.getProtocol().equals("jar")) {
							// The resource is contain into a Jar
							try {
								String fileName = url.getPath().substring(0, url.getPath().indexOf("!"));
								fileName = fileName.substring(5);
								JarFile jarFile = new JarFile(fileName);
								inputStream = jarFile.getInputStream(jarFile.getEntry(url.getPath().substring(url.getPath().indexOf("!") + 2)));
							}
							catch (IOException e) {
								inputStream = null;
							}
						}
					}
					catch (FileNotFoundException e) {
						inputStream = null;
					}
				}
			}
		}
		// self first
		else {
			URL url = findResource(resourceName);
			if (url == null && !isIsolated()) {
				url = parent.getResource(resourceName);
			}
			if (url != null) {
				try {
					if (url.getProtocol().equals("file")) {
						// The resource is a file
						inputStream = new FileInputStream(url.getPath());
					}
					else if (url.getProtocol().equals("jar")) {
						// The resource is contain into a Jar
						try {
							String fileName = url.getPath().substring(0, url.getPath().indexOf("!"));
							fileName = fileName.substring(5);
							JarFile jarFile = new JarFile(fileName);
							inputStream = jarFile.getInputStream(jarFile.getEntry(url.getPath().substring(url.getPath().indexOf("!") + 2)));
						}
						catch (IOException e) {
							inputStream = null;
						}
					}
				}
				catch (FileNotFoundException e) {
					inputStream = null;
				}
			}
		}
		return inputStream;
	}

	/**
     * Return a enumeration of resources
     * 
     * @see java.lang.ClassLoader#getResources(java.lang.String)
     */
	@Override
	public Enumeration<URL> getResources(String resourceName) throws IOException {
		Enumeration<URL> resources = null;
		// parent first
		if (useParentFirst(resourceName, false)) {
			resources = parent.getResources(resourceName);
			if (resources == null) {
				resources = findResources(resourceName);
			}
		}
		// self first
		else {
			resources = findResources(resourceName);
			if (!resources.hasMoreElements() && !isIsolated()) {
				resources = parent.getResources(resourceName);
			}
		}
		return resources;
	}

	// ----------------------------------------------------------------------
	// implementation specific methods
	// ----------------------------------------------------------------------

	/**
     * Initialize the petals class loader
     * 
     * @param classpathLocations elements of the classpath
     */
	protected void initialize(List<String> classpathLocations) throws IOException {
		this.parentFirst = true;
		this.ignoreParent = false;
		this.usePackageIntelligence = false;
		if (parent == null) {
			parent = getSystemClassLoader();
		}
		factories = new URLFactory[bases.length];
		locators = new Locator[bases.length];
		// Create factories and locator for each base URL
		for (int i = 0; i < bases.length; i++) {
			factories[i] = URLFactory.getFactory(bases[i]);
			locators[i] = Locator.getLocator(bases[i]);
		}
		// Add all classpathLocations into the repository
		for (String location : classpathLocations) {
			addInRepository(location);
		}
	}

	/**
     * Method to know if we choose to use the parent class loader first
     * 
     * @param resourceName resource name
     * @param usingForClass resource as a class
     * @return boolean
     */
	protected boolean useParentFirst(String resourceName, boolean usingForClass) {
		// begin with the chosen delegation model
		boolean useParentFirst = isParentFirst() && !isIsolated();
		// try self-first based on intelligent mechanism of known packages
		if (useParentFirst) {
			if (isUsingPackageIntelligence() && usingForClass) {
				String packageName = getPackageName(resourceName);
				// self-first <-- when parent first is set to false
				useParentFirst = !knowLocalPackage(packageName);
			}
		}
		// always use parent class loader when loading java packages
		else {
			useParentFirst = (JAVA_PACKAGE.startsWith(resourceName) || JAVAX_PACKAGE.startsWith(resourceName));
		}
		return useParentFirst;
	}

	/**
     * Add a package name to the known package
     * 
     * @param packageName package name
     */
	protected void addLocalKnownPackage(String packageName) {
		if (localKnownPackages != null) {
			localKnownPackages.add(packageName);
		}
	}

	/**
     * Check if a package is in the known package list
     * 
     * @param packageName package name
     * @return true if the package is already known
     */
	protected boolean knowLocalPackage(String packageName) {
		boolean known = false;
		if (localKnownPackages != null) {
			known = localKnownPackages.contains(packageName);
		}
		return known;
	}

	/**
     * Return the package name of a class
     * 
     * @param className complete name of a class
     * @return package name
     */
	protected String getPackageName(String className) {
		int lastSeparator = className.lastIndexOf('.');
		String packageName = className.substring(0, lastSeparator);
		return packageName;
	}

	/**
     * Add specified location into the repository. If location is found in
     * multiple URLs of the bases, new URLs will be added multiple times.
     * 
     * @param location an entry name (for a jar) or a path name (for a
     *        directory)
     * @throws IOException when constructed URL is malformed
     */
	protected void addInRepository(String location) throws IOException {
		// test existence in each base URL
		for (int i = 0; i < bases.length; i++) {
			if (locators[i].hasDirectory(location) || locators[i].hasFile(location)) {
				addURL(factories[i].getURL(location));
			}
		}
	}

	/**
     * @return Returns a String representation of the PetalsClassLoader
     */
	public String toString() {
		StringBuffer sb = new StringBuffer();

		sb.append("classloader : " + getClass().getName() + "\n");
		sb.append("\tmodules bases (not in loader!) : \n");
		for (int i = 0; i < bases.length; i++) {
			sb.append("\t\t -" + bases[i] + "\n");
		}
		sb.append("\trepositories :\n");
		URL[] rep = getURLs();
		for (int i = 0; i < rep.length; i++) {
			sb.append("\t\t -" + rep[i] + "\n");
		}
		sb.append("\tparent : " + getParent() + "\n");

		return sb.toString();
	}

	/**
     * Expose the protected <code>findLoadedClass</code> method. Used by the
     * ShareLibraries class loader
     * 
     * @param className
     * @return
     * @throws ClassNotFoundException
     */
	Class<?> findLoadedClazz(String className) throws ClassNotFoundException {
		return super.findLoadedClass(className);
	}
}
