/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 Fossil E-Commerce, http://www.fossilec.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: SharedLibrariesClassLoader.java 216 2006-04-13 14:18:14Z ofabre $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.classloader;

import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

/**
 * This classLoader englobes all SharedLibraries classLoader. When the
 * ClassLoader of a component does not found a class, This classLoader search in
 * all the SLClassLoader the class. If the class is not found in those
 * SLClassLoader, search in the parent classLoader.
 * 
 * @version $Rev: 216 $ $Date: 2006-04-13 16:18:14 +0200 (jeu., 13 avr. 2006) $
 * @since Petals 1.0
 * @author Adrien LOUIS
 * @author ddesjardins - eBMWebsourcing
 */
public class SharedLibrariesClassLoader extends ClassLoader {

	protected Map<String, PetalsClassLoader> slClassLoaders;

	/**
	 * Parent Class Loader.
	 */
	protected ClassLoader parent;

	/**
	 * Create a SharedLibrariesClassLoader, with SystemClassLoader as parent.
	 * 
	 * @throws IOException
	 */
	public SharedLibrariesClassLoader() throws IOException {
		super();
		parent = getSystemClassLoader();
		slClassLoaders = Collections.synchronizedMap(new HashMap<String, PetalsClassLoader>());
	}

	public void addSharedLibrariesClassLoader(String id, PetalsClassLoader cl) {
		cl.setParentFirst(false);
		slClassLoaders.put(id, cl);
	}

	public void removeSharedLibrariesClassLoader(String id) {
		slClassLoaders.remove(id);
	}

	// ----------------------------------------------------------------------
	// overriding methods
	// ----------------------------------------------------------------------

	/**
	 * Search in all referenced SharedLibrariesClassLoaders for the requested
	 * class. If the class is not found, search in the parent classLoader.
	 * 
	 * @see java.lang.ClassLoader#loadClass(java.lang.String, boolean)
	 */
	protected synchronized Class<?> loadClass(String className, boolean resolve, List<String> sharedLibrariesNameList) throws ClassNotFoundException {
		Class<?> clazz = null;
		clazz = findLoadedClass(className, sharedLibrariesNameList);
		if (clazz == null) {
			// self first
			try {
				clazz = findClass(className, sharedLibrariesNameList);
			}
			catch (ClassNotFoundException cnfe) {
				clazz = parent.loadClass(className);
			}
		}
		if (resolve) {
			resolveClass(clazz);
		}
		return clazz;
	}

	/**
	 * Search in all referenced SharedLibrariesClassLoaders for the requested
	 * resource.
	 * 
	 */
	protected synchronized URL getResource(String resourceName, List<String> sharedLibrariesNameList) {
		URL url = null;
		for (Iterator iter = sharedLibrariesNameList.iterator(); iter.hasNext() && url == null;) {
			PetalsClassLoader element = (PetalsClassLoader) slClassLoaders.get(iter.next());
			if (element != null) {
				url = element.getResource(resourceName);
			}
		}
		return url;
	}

	/**
	 * Search in all referenced SharedLibrariesClassLoaders for the requested
	 * resource.
	 * 
	 */
	protected synchronized InputStream getResourceAsStream(String resourceName, List<String> sharedLibrariesNameList) {
		InputStream inputStream = null;
		for (Iterator iter = sharedLibrariesNameList.iterator(); iter.hasNext() && inputStream == null;) {
			PetalsClassLoader element = (PetalsClassLoader) slClassLoaders.get(iter.next());
			if (element != null) {
				inputStream = element.getResourceAsStream(resourceName);
			}
		}
		return inputStream;
	}

	/**
	 * Ask to each referenced SLClassLoader (which name is specified in the
	 * list) to find the class. If none of the SLclassLoader knows this class,
	 * throw an exception
	 * 
	 * @throws ClassNotFoundException
	 * @see java.net.URLClassLoader#findClass(java.lang.String)
	 */
	protected Class<?> findClass(String className, List<String> sharedLibrariesNameList) throws ClassNotFoundException {
		Class result = null;
		if (sharedLibrariesNameList != null) {
			for (Iterator iter = sharedLibrariesNameList.iterator(); iter.hasNext() && result == null;) {
				PetalsClassLoader element = (PetalsClassLoader) slClassLoaders.get(iter.next());
				if (element != null) {
					try {
						result = element.findClass(className);
					}
					catch (ClassNotFoundException e) {
						// the class is not found with this class loader,
						// search in the next one
					}
				}
			}
		}
		if (result == null) {
			throw new ClassNotFoundException();
		}
		return result;
	}

	protected Class<?> findLoadedClass(String className, List<String> sharedLibrariesNameList) {
		Class result = null;
		if (sharedLibrariesNameList != null) {
			for (Iterator iter = sharedLibrariesNameList.iterator(); iter.hasNext() && result == null;) {
				PetalsClassLoader element = (PetalsClassLoader) slClassLoaders.get(iter.next());
				if (element != null) {
					try {
						result = element.findLoadedClazz(className);
					}
					catch (ClassNotFoundException e) {
						// the class is not found with this class loader,
						// search in the next one
					}
				}
			}
		}
		return result;
	}

}
