/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 Fossil E-Commerce, http://www.fossilec.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: RepositoryImpl.java 250 2006-04-21 12:20:57Z ofabre $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.repository;

import java.io.File;
import java.io.IOException;
import java.net.URI;
import java.util.zip.ZipException;
import java.util.zip.ZipFile;

import org.objectweb.petals.PetalsException;
import org.objectweb.petals.util.FileUtil;
import org.objectweb.petals.util.SystemUtil;
import org.objectweb.petals.util.ZipUtil;

/**
 * Platform Component Repository service implementation.
 *
 * @version	$Rev: 250 $ $Date: 2006-04-21 12:20:57Z $
 * @since	Petals 1.0
 * @author	<a href="mailto:rmarins@fossilec.com">Rafael Marins</a>
 */
public class RepositoryImpl implements RepositoryService {

	/**
	 * Path to repository location within Petals installation directory.
	 */
	private final static String REPOSITORY_PATH = "repository";

	/**
	 * Path to component install directory
	 */
	private final static String INSTALL_PATH = "install";

	/**
	 * Path to component working directory
	 */
	private final static String WORK_PATH = "work";

	
	/**
	 * The complete repository directory path.
	 */
	private static String repositoryDirectory = null;

	
	// ----------------------------------------------------------------------
	//   RepositoryService interface implementation
	// ----------------------------------------------------------------------

	/**
	 * @see org.objectweb.petals.repository.RepositoryService#containsPackage(java.lang.String)
	 */
	public boolean containsPackage(String uuid) {

		File installationRoot = getComponentRoot(uuid);

		boolean contain = (installationRoot.exists() && installationRoot.isDirectory());

		return contain;
	}

	/**
	 * @see org.objectweb.petals.repository.RepositoryService#addPackage(String, java.net.URI)
	 */
	public URI addPackage(String uuid, URI zipArchiveLocation)
			throws PetalsException {

		// don't care about replacing existing data
		if (containsPackage(uuid)) {
			removePackage(uuid);
		}

		File installationRoot = new File(getComponentRoot(uuid),INSTALL_PATH);
		
		File workRoot = new File(new File(installationRoot,".."),WORK_PATH);
		

		// recreate dirs necessary for adding the package into the repository
		if (!installationRoot.mkdirs()) {
			throw new PetalsException(
                    "Failed to create installation root dir: " +
                    installationRoot.getAbsolutePath());
		}
		// recreate dirs necessary for adding the package into the repository
		if (!workRoot.mkdirs()) {
			throw new PetalsException(
                    "Failed to create installation root dir: " +
                    workRoot.getAbsolutePath());
		}

		// explode package Zip archive file into repository
		ZipFile zipArchive = null;

		try {
			File archiveFile = new File(zipArchiveLocation);
			zipArchive = new ZipFile(archiveFile);
		} catch (ZipException ze) {
			throw new PetalsException("Unable to open Zip archive file: " +
					zipArchiveLocation.toString(), ze);
		} catch (IOException ioe) {
			throw new PetalsException("Unexpected exception.", ioe);
		}

		ZipUtil.explodeIntoDirectory(zipArchive, installationRoot);
		return installationRoot.toURI();

	}
	
	/**
	 * @see org.objectweb.petals.repository.RepositoryService#explodeSUIntoSAInstallFolder(String, URI, String)
	 */
	public URI explodeSUIntoSAInstallFolder(String suId, 
								URI suZipLocation, 
								String saId)
			throws PetalsException {

		File installationRoot = new File(getComponentInstallRoot(saId),suId);

		// recreate dirs necessary for adding the package into the repository
		if (!installationRoot.mkdirs()) {
			throw new PetalsException(
                    "Failed to create installation root dir: " +
                    installationRoot.getAbsolutePath());
		}

		// explode package Zip archive file into repository
		ZipFile zipArchive = null;

		try {
			File archiveFile = new File(suZipLocation);
			zipArchive = new ZipFile(archiveFile);
		} catch (ZipException ze) {
			throw new PetalsException("Unable to open Zip archive file: " +
					suZipLocation.toString(), ze);
		} catch (IOException ioe) {
			throw new PetalsException("Unexpected exception.", ioe);
		}

		ZipUtil.explodeIntoDirectory(zipArchive, installationRoot);
		return installationRoot.toURI();

	}

	/**
	 * @see org.objectweb.petals.repository.RepositoryService#removePackage(java.lang.String)
	 */
	public boolean removePackage(String componentId) {

        boolean removed = true;
		File installationRoot = getComponentRoot(componentId);

		// adding component don't care about replacing existing data
		if (installationRoot.exists()) {
			removed = FileUtil.removeDirectory(installationRoot);
		}

        return removed;
	}

	// ----------------------------------------------------------------------
	//   implementation specific methods
	// ----------------------------------------------------------------------

	private String getRepositoryDirectory() {
		if (repositoryDirectory == null) {
			repositoryDirectory = SystemUtil.getPetalsInstallDirectory()
					.getAbsolutePath() + File.separator + REPOSITORY_PATH;
		}
		return repositoryDirectory;
	}

	public File getComponentRoot(String componentId) {

		String path = getRepositoryDirectory() + File.separator + componentId;

		File installationRoot = new File(path);

		return installationRoot;
	}
	
	public File getComponentInstallRoot(String componentId) {

		String path = getRepositoryDirectory() + File.separator + componentId + File.separator + INSTALL_PATH;

		File installationRoot = new File(path);

		return installationRoot;
	}

	public File getComponentWorkRoot(String componentId) {

		String path = getRepositoryDirectory() + File.separator + componentId+ File.separator +WORK_PATH;

		File installationRoot = new File(path);

		return installationRoot;
	}

}
